
  ----------------------------------------------------------------------------
  --                                                                        --
  --  Life on Mars simulation:                                              --
  --  Copyright (C) 2008-2011 J.Simpson, C.G.Ritson, P.H.Welch and D.C.Wood --
  --                                                                        --
  --  This program is free software; you can redistribute it and/or         --
  --  modify it under the terms of the GNU General Public License           --
  --  as published by the Free Software Foundation, either version 3        --
  --  of the License, or (at your option) any later version.                --
  --                                                                        --
  --  This program is distributed in the hope that it will be               --
  --  useful, but WITHOUT ANY WARRANTY; without even the implied            --
  --  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR               --
  --  PURPOSE. See the GNU Lesser General Public License for more           --
  --  details.                                                              --
  --                                                                        --
  --  You should have received a copy of the GNU General Public License     --
  --  along with this library; if not, see <http://www.gnu.org/licenses/>.  --
  --                                                                        --
  --  Author contact: P.H.Welch@kent.ac.uk                                  --
  --                                                                        --
  ----------------------------------------------------------------------------


#IF NOT DEFINED (MARS.ROBOT.INC)
#DEFINE MARS.ROBOT.INC

-- camera is unreliable while turning
-- camera scan takes, two seconds ?
VAL INT CAMERA.FOV        IS 90:      -- field-of-view (degrees)
VAL INT CAMERA.SCAN.TIME  IS 2000000:

VAL INT GRIPPER.DEPLOY.TIME IS 5000000:

DATA TYPE BLOB
  RECORD
    INT colour:
    REAL32 bearing:  -- degrees
    REAL32 size:     -- 0 to 1
:

PROTOCOL P.CAMERA.DATA IS BLOB:

PROTOCOL P.GRIPPER.CMD
  CASE
    deploy
:

------------------------
--        Front      
--          0
--          |
-- Left --     -- Right
--          |
--         180
--        Back
------------------------
VAL INT FRONT.LEFT  IS 0:
VAL INT FRONT.RIGHT IS 1:
VAL INT BACK.RIGHT  IS 2:
VAL INT BACK.LEFT   IS 3:

PROTOCOL P.HAZARD.DETECTOR IS INT:

VAL INT HAZARD.SECTIONS IS 4:

PROTOCOL P.MOTOR.CMD
  CASE
    drive.forward
    drive.backward
    turn.left
    turn.right
    stop
:
PROTOCOL P.MOTOR.FEEDBACK IS INT: -- millimeters / degrees 

PROTOCOL P.OPERATOR.REQUEST
  CASE
    turn; INT               -- relative heading (degrees)
    move; INT               -- relative distance (millimeters)
    find.blob; INT          -- colour
    deploy.sensor; INT; INT -- marker colours
:

PROTOCOL P.OPERATOR.RESPONSE
  CASE
    turn.complete; INT      -- approx. amount turned
    move.complete; INT      -- approx. amount moved
    hazard.detected; INT    -- approx. amount moved
    blob.found; INT; BLOB   -- new heading; blob data
    blob.not.found; INT     -- new heading
    sensor.deployed
    drop.zone.not.found
:

INT FUNCTION IABS (VAL INT v)
  INT result:
  VALOF
    IF
      v >= 0
        result := v
      TRUE
        result := -v
    RESULT result
:

INT FUNCTION array.minimum (VAL []INT array)
  INT min:
  VALOF
    SEQ
      min := array[0]
      SEQ i = 1 FOR (SIZE array) - 1
        IF
          array[i] < min
            min := array[i]
          TRUE
            SKIP
    RESULT min
:

#ENDIF -- !MARS.ROBOT.INC

